/**
 * @license angular-gamification
 * MIT license
 */

import { Inject, Injectable, NgModule } from '@angular/core';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class MissionService {
    /**
     * @param {?} name
     * @param {?} points
     * @param {?=} description
     * @param {?=} startFn
     * @param {?=} achieveFn
     */
    constructor(name, points, description, startFn, achieveFn) {
        this.name = 'Mission';
        this.points = 0;
        this.description = '';
        this.startFn = () => { };
        this.achieveFn = () => { };
        this.name = name;
        this.points = points;
        this.description = description || this.description;
        this.startFn = startFn || this.startFn;
        this.achieveFn = achieveFn || this.achieveFn;
    }
    /**
     * @param {?} points
     * @return {?}
     */
    setPoints(points) {
        this.points = points;
    }
    /**
     * @return {?}
     */
    start() {
        this.startFn();
    }
    /**
     * @param {?} report
     * @return {?}
     */
    achieve(report) {
        this.achieveFn(report);
    }
}
MissionService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
MissionService.ctorParameters = () => [
    null,
    null,
    null,
    { type: Function, },
    { type: Function, },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ProgressBarService {
    /**
     * @param {?} maxPoints
     * @param {?=} updateFn
     * @param {?=} startFn
     */
    constructor(maxPoints, updateFn, startFn) {
        this.maxPoints = 0;
        this.updateFn = () => { };
        this.startFn = () => { };
        this.maxPoints = maxPoints || this.maxPoints;
        this.updateFn = updateFn || this.updateFn;
        this.startFn = startFn || this.startFn;
        this.isFull = false;
        // return this;
    }
    /**
     * @param {?} points
     * @return {?}
     */
    update(points) {
        if (!this.isFull) {
            this.updateFn();
            if (this.maxPoints <= points) {
                this.isFull = true;
            }
        }
    }
    /**
     * @return {?}
     */
    start() {
        this.startFn(this.maxPoints);
    }
    /**
     * @param {?} points
     * @return {?}
     */
    getProgress(points) {
        return Math.min(Math.max(Math.ceil(100 * points / this.maxPoints), 0), 100);
    }
}
ProgressBarService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
ProgressBarService.ctorParameters = () => [
    null,
    { type: Function, },
    { type: Function, },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class BreakpointService {
    /**
     * @param {?} pointsThreshold
     * @param {?=} action
     */
    constructor(pointsThreshold, action) {
        this.actionFunc = () => { };
        this.pointsThreshold = pointsThreshold;
        this.actionFunc = action || this.actionFunc;
    }
    /**
     * @return {?}
     */
    action() {
        this.actionFunc();
    }
}
BreakpointService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
BreakpointService.ctorParameters = () => [
    null,
    { type: Function, },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class ReportService {
    constructor() {
        this.reports = [];
    }
    /**
     * @param {?} reports
     * @return {?}
     */
    set(reports) {
        this.reports = reports;
    }
    /**
     * @param {?} report
     * @return {?}
     */
    add(report) {
        this.reports.push(report);
        return report;
    }
    /**
     * @return {?}
     */
    get() {
        return /** @type {?} */ (this.reports);
    }
    /**
     * @return {?}
     */
    clear() {
        this.reports = [];
    }
}
ReportService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
ReportService.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
class GamificationService {
    /**
     * @param {?} config
     */
    constructor(config) {
        this.config = config;
        this.points = 0;
        this.levels = [];
        this.breakpoints = Array();
        this.missions = Array();
        this.components = Array();
        this.reportService = new ReportService();
        this.levels = config.levels || this.levels;
        this.level = config.level || this.levels[0];
        this.points = config.points || this.points;
    }
    /**
     * @param {?} points
     * @param {?=} updateFn
     * @param {?=} startFn
     * @return {?}
     */
    addComponent(points, updateFn, startFn) {
        const /** @type {?} */ component = new ProgressBarService(points, updateFn, startFn);
        this.components.push(component);
        component.start();
        return component;
    }
    /**
     * @return {?}
     */
    getComponents() {
        return this.components;
    }
    /**
     * @param {?} name
     * @param {?} points
     * @param {?=} description
     * @param {?=} startFn
     * @param {?=} achieveFn
     * @return {?}
     */
    addMission(name, points, description, startFn, achieveFn) {
        const /** @type {?} */ mission = new MissionService(name, points, description, startFn, achieveFn);
        this.missions.push(mission);
        mission.start();
        return mission.name;
    }
    /**
     * @return {?}
     */
    getMissions() {
        return this.missions;
    }
    /**
     * @param {?} points
     * @param {?=} actionFn
     * @return {?}
     */
    addBreakpoint(points, actionFn) {
        const /** @type {?} */ breakpoint = new BreakpointService(points, actionFn);
        this.breakpoints.push(breakpoint);
    }
    /**
     * @return {?}
     */
    getBreakpoints() {
        return this.breakpoints;
    }
    /**
     * @param {?} points
     * @return {?}
     */
    addPoints(points) {
        this.points += points;
        this.breakpoints.forEach((breakpoint, index) => {
            if (((this.points - points) < this.breakpoints[index].pointsThreshold) && (this.breakpoints[index].pointsThreshold <= (this.points))) {
                this.breakpoints[index].action();
            }
        });
        this.components.forEach((component, index) => {
            this.components[index].update(points);
        });
    }
    /**
     * @return {?}
     */
    getPoints() {
        return this.points;
    }
    /**
     * @param {?} points
     * @return {?}
     */
    setPoints(points) {
        this.points = points;
    }
    /**
     * @param {?} levels
     * @return {?}
     */
    setLevels(levels) {
        this.levels = levels;
    }
    /**
     * @return {?}
     */
    getLevels() {
        return this.levels;
    }
    /**
     * @param {?} level
     * @return {?}
     */
    addLevel(level) {
        this.levels.push(level);
    }
    /**
     * @param {?} level
     * @return {?}
     */
    setLevel(level) {
        this.level = level;
    }
    /**
     * @return {?}
     */
    getLevel() {
        return this.level;
    }
    /**
     * @param {?} points
     * @return {?}
     */
    getLevelByPoints(points) {
        return this.getLevels().filter(level => {
            if (level.range) {
                return (level.range.min <= points) && (level.range.max >= points);
            }
        }, this.setLevel)[0];
    }
    /**
     * @param {?} reports
     * @return {?}
     */
    setReports(reports) {
        this.reportService.set(reports);
    }
    /**
     * @return {?}
     */
    getReports() {
        return this.reportService.get();
    }
    /**
     * @return {?}
     */
    clearReport() {
        this.reportService.clear();
    }
    /**
     * @param {?} name
     * @return {?}
     */
    achieveMission(name) {
        this.missions.map((mission, index) => {
            if (mission.name === name) {
                this.addPoints(mission.points);
                let /** @type {?} */ report = this.reportService.add({
                    description: mission.description,
                    points: mission.points
                });
                mission.achieve(report);
            }
        });
    }
}
GamificationService.decorators = [
    { type: Injectable },
];
/** @nocollapse */
GamificationService.ctorParameters = () => [
    { type: undefined, decorators: [{ type: Inject, args: ['config',] },] },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
// Consider registering providers using a forRoot() method
// when the module exports components, directives or pipes that require sharing the same providers instances.
// Consider registering providers also using a forChild() method
// when they requires new providers instances or different providers in child modules.
class GamificationModule {
    /**
     * Use in AppModule: new instance of GamificationService.
     * @param {?=} config
     * @return {?}
     */
    static forRoot(config) {
        return {
            ngModule: GamificationModule,
            providers: [
                GamificationService,
                {
                    provide: 'config', useValue: config
                }
                // {
                //   provide: GamificationService, useValue: config, useClass: GamificationModule.forRoot(config)
                // }
            ]
        };
    }
    /**
     * Use in features modules with lazy loading: new instance of GamificationService.
     * @param {?=} config
     * @return {?}
     */
    static forChild(config) {
        return {
            ngModule: GamificationModule,
            providers: [
                GamificationService,
                {
                    provide: 'config', useValue: config
                }
                // GamificationService,
                // {
                //   provide: GamificationService, useValue: config, useClass: GamificationService
                // }
            ]
        };
    }
}
GamificationModule.decorators = [
    { type: NgModule, args: [{
                declarations: [],
                exports: []
            },] },
];
/** @nocollapse */
GamificationModule.ctorParameters = () => [];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
// Public classes.

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Angular Gamification Library
 * Improve the interaction of your users with your application using gamification
 * Copyright Guilherme Siquinelli (Gui Seek <guiseek@gmail.com>)
 * MIT license
 * https://github.com/guiseek/angular-gamification
 */
/**
 * Entry point for all public APIs of the package.
 */

// This file only reexports content of the `src` folder. Keep it that way.

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Generated bundle index. Do not edit.
 */

export { GamificationService, GamificationModule };
//# sourceMappingURL=angular-gamification.js.map
